import { isDefined } from '@collectable/core';
import { RedBlackTreeIterator, PathNode, findNodeByKey, findMaxNodeLeftOfKey, findMinNodeRightOfKey, findPathToNodeByKey, findPathToMaxNodeLeftOfKey, findPathToMinNodeRightOfKey } from '../internals';
export function find(op, key, tree) {
    var exact = false;
    switch (op) {
        case 'gte': exact = true;
        case 'gt':
            return findMinNodeRightOfKey(exact, key, tree._root, tree._compare);
        case 'lte': exact = true;
        case 'lt':
            return findMaxNodeLeftOfKey(exact, key, tree);
        case 'eq':
            return findNodeByKey(key, tree);
    }
    throw new Error(`Invalid find operation; must be 'lt', 'lte', 'gt', 'gte' or 'eq'`);
}
/**
 * Returns the key of whichever entry in the tree which is closest to the specified input key. The logic determining
 * which entry to locate is controlled by the `op` parameter.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {FindOp} op The operation that determines which entry to find in the tree
 * @param {K} key A reference key used as input to the find operation
 * @param {RedBlackTree<K, V>} tree The input tree
 * @returns {(K|undefined)} The key of the matched entry, or undefined if no matching entry was found
 */
export function findKey(op, key, tree) {
    const node = find(op, key, tree);
    return isDefined(node) ? node.key : void 0;
}
/**
 * Returns the value of whichever entry in the tree which is closest to the specified input key. The logic determining
 * which entry to locate is controlled by the `op` parameter.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {FindOp} op The operation that determines which entry to find in the tree
 * @param {K} key A reference key used as input to the find operation
 * @param {RedBlackTree<K, V>} tree The input tree
 * @returns {(V|undefined)} The value of the matched entry, or undefined if no matching entry was found
 */
export function findValue(op, key, tree) {
    const node = find(op, key, tree);
    return isDefined(node) ? node.value : void 0;
}
export function iterateFrom(op, reverse, key, tree) {
    var exact = false, path;
    switch (op) {
        case 'gte':
            exact = true;
        case 'gt':
            path = findPathToMinNodeRightOfKey(exact, key, tree._root, PathNode.NONE, tree._compare);
            break;
        case 'lte':
            exact = true;
        case 'lt':
            path = findPathToMaxNodeLeftOfKey(exact, key, tree._root, PathNode.NONE, tree._compare);
            break;
        case 'eq':
            path = findPathToNodeByKey(key, tree._root, tree._compare);
            break;
        default:
            throw new Error(`Invalid find operation; must be 'lt', 'lte', 'gt', 'gte' or 'eq'`);
    }
    return new RedBlackTreeIterator(isDefined(path) ? path : PathNode.NONE, reverse);
}

//# sourceMappingURL=find.js.map
